package com.example.julian.endlessroll.entities;

import com.example.julian.endlessroll.data.Vector;
import com.example.julian.endlessroll.entities.textures.Texture;
import com.example.julian.endlessroll.levels.Gap;
import com.example.julian.endlessroll.levels.TerrainData;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Julian on 08.12.2015.
 */
public class Terrain extends ArrayList<TerrainTile> {

    private Texture texture;
    private float height;
    private boolean endless;

    public Terrain(Texture texture) {
        this.texture = texture;
    }

    private void create(float endX, float height, List<Gap> gaps) {
        this.height = height;
        float y = calculateYPositionFromHeight();
        float h = 1 + height;
        float lastGapRightEdge = -3;
        for (int i = 0; i < gaps.size() + 1; i++) {
            float nextGapLeftEdge;
            if (i == gaps.size())
                nextGapLeftEdge = endX;
            else
                nextGapLeftEdge = gaps.get(i).getLeftEdge();
            float width = nextGapLeftEdge - lastGapRightEdge;
            float x = lastGapRightEdge + width / 2;
            TerrainTile entity = new TerrainTile(texture, new Vector(x, y), width, h);
            super.add(entity);
            if (i != gaps.size())
                lastGapRightEdge = gaps.get(i).getRightEdge();
        }
    }

    public void loadData(TerrainData data) {
        super.clear();
        this.endless = false;
        create(data.getEndX(), data.getHeight(), data.getGaps());
    }

    public void createEndless(float height) {
        super.clear();
        this.endless = true;
        this.height = height;
        super.add(new TerrainTile(texture, new Vector(0, calculateYPositionFromHeight()), 5, 1 + height));
    }

    public void update(float cameraX) {
        if (!super.isEmpty()) {
            if (super.get(0).getRightEdge() - cameraX < -3) {
                super.remove(0);
            }
            if (endless) {
                Entity last = super.get(super.size() - 1);
                if (last.getRightEdge() - cameraX < 3)
                    super.add(new TerrainTile(texture, new Vector(last.getRightEdge() + 2.5f, calculateYPositionFromHeight()), 5, 1 + height));
            }
        }
    }

    private float calculateYPositionFromHeight() {
        return height - (1 + height) / 2;
    }

    public float getHeight() {
        return height;
    }

}
