package com.example.julian.endlessroll.entities.collision;

import com.example.julian.endlessroll.data.Vector;
import com.example.julian.endlessroll.entities.Entity;
import com.example.julian.endlessroll.entities.Player;
import com.example.julian.endlessroll.entities.collision.collisionData.EntityCollisionData;
import com.example.julian.endlessroll.entities.collision.geometry.Circle;
import com.example.julian.endlessroll.entities.collision.geometry.Geometry;
import com.example.julian.endlessroll.entities.collision.geometry.Quad;
import com.example.julian.endlessroll.entities.collision.geometry.Triangle;

/**
 * Created by Julian on 01.12.2015.
 */
public class CollisionDetector {

    private CircleTriangleCollisionDetector triangleDetector;

    public CollisionDetector() {
        triangleDetector = new CircleTriangleCollisionDetector();
    }

    public boolean quadQuadCollision(Quad q1, Quad q2) {
        float xDistance = Math.abs(q2.getPosition().x - q1.getPosition().x);
        float yDistance = Math.abs(q2.getPosition().y - q1.getPosition().y);
        if (xDistance >= q1.getWidth() / 2 + q2.getWidth() / 2)
            return false;
        if (yDistance >= q1.getHeight() / 2 + q2.getHeight() / 2)
            return false;
        return true;
    }

    public EntityCollisionData playerEntityCollision(Player player, Entity entity) {
        Circle circle = new Circle(player);
        Quad quad = entity;
        if (circleQuadCollision(circle, quad)) {
            Edge edge = circleQuadCollisionEdge(circle, player.getMovement(), quad, entity.getMovement());
            return new EntityCollisionData(entity, edge);
        }
        return new EntityCollisionData(null, null);
    }

    public boolean circleToolCollision(Circle circle, Geometry bounds) {
        if (bounds instanceof Circle)
            return circleCircleCollision(circle, (Circle) bounds);
        else if (bounds instanceof Quad)
            return circleQuadCollision(circle, (Quad) bounds);
        else if (bounds instanceof Triangle)
            return triangleDetector.circleTriangleCollision(circle, (Triangle) bounds);
        return false;
    }

    private boolean circleCircleCollision(Circle c1, Circle c2) {
        float distance = Math.abs(c1.getCenter().vectorTo(c2.getCenter()).length());
        float radiusSum = c1.getRadius() + c2.getRadius();
        return distance < radiusSum;
    }

    private boolean circleQuadCollision(Circle circle, Quad quad) {
        Vector distance = circle.getCenter().vectorTo(quad.getPosition());
        distance.x = Math.abs(distance.x);
        distance.y = Math.abs(distance.y);

        if (distance.x > circle.getRadius() + quad.getWidth() / 2) return false;
        if (distance.y > circle.getRadius() + quad.getHeight() / 2) return false;

        if (distance.x <= quad.getWidth() / 2) return true;
        if (distance.y <= quad.getHeight() / 2) return true;

        float cornerDistance_sq = (distance.x - quad.getWidth() / 2) * (distance.x - quad.getWidth() / 2) + (distance.y - quad.getHeight() / 2) * (distance.y - quad.getHeight() / 2);
        return cornerDistance_sq <= circle.getRadius() * circle.getRadius();
    }

    private Edge circleQuadCollisionEdge(Circle circle, Vector circleMovement, Quad quad, Vector quadMovement) {
        Vector relativeMovement = circleMovement.translate(quadMovement);

        //LEFT || TOP || BOTTOM
        if (relativeMovement.x > 0) {
            //LEFT || BOTTOM
            if (relativeMovement.y > 0) {
                float toLeftDistance = quad.getLeftEdge() - circle.getCenter().x;
                float actualY = toLeftDistance * (relativeMovement.y / relativeMovement.x) + circle.getCenter().y;
                if (actualY < quad.getBottomEdge())
                    return Edge.BOTTOM;
                else
                    return Edge.LEFT;
            }
            //LEFT || TOP
            else if (relativeMovement.y < 0) {
                float toLeftDistance = quad.getLeftEdge() - circle.getCenter().x;
                float actualY = toLeftDistance * (relativeMovement.y / relativeMovement.x) + circle.getCenter().y;
                if (actualY < quad.getTopEdge())
                    return Edge.LEFT;
                else
                    return Edge.TOP;
            } else return Edge.LEFT;
        }
        //RIGHT || TOP || BOTTOM
        else if (relativeMovement.x < 0) {
            //RIGHT || BOTTOM
            if (relativeMovement.y > 0) {
                float toRightDistance = quad.getRightEdge() - circle.getCenter().x;
                float actualY = toRightDistance * (relativeMovement.y / relativeMovement.x) + circle.getCenter().y;
                if (actualY < quad.getBottomEdge())
                    return Edge.BOTTOM;
                else
                    return Edge.RIGHT;
            }
            //RIGHT || TOP
            else if (relativeMovement.y < 0) {
                float toRightDistance = quad.getRightEdge() - circle.getCenter().x;
                float actualY = toRightDistance * (relativeMovement.y / relativeMovement.x) + circle.getCenter().y;
                if (actualY < quad.getTopEdge())
                    return Edge.RIGHT;
                else
                    return Edge.TOP;
            } else return Edge.RIGHT;
        } else {
            if (relativeMovement.y > 0)
                return Edge.BOTTOM;
            else if (relativeMovement.y < 0)
                return Edge.TOP;
        }
        return Edge.NONE;
    }
}
