package com.example.julian.endlessroll.entities.collision;

import com.example.julian.endlessroll.entities.Player;
import com.example.julian.endlessroll.entities.Star;
import com.example.julian.endlessroll.entities.collision.collisionData.EntityCollisionData;
import com.example.julian.endlessroll.entities.collision.collisionData.ObstacleCollisionData;
import com.example.julian.endlessroll.entities.collision.collisionData.PlayerCollisionData;
import com.example.julian.endlessroll.entities.collision.collisionData.ToolCollisionData;
import com.example.julian.endlessroll.entities.tools.Tool;
import com.example.julian.endlessroll.main.game.Game;
import com.example.julian.endlessroll.main.game.GameScene;
import com.example.julian.endlessroll.main.game.Physics;

/**
 * Created by Julian on 02.01.2016.
 */
public class CollisionManager {

    private Game game;
    private Player player;

    public CollisionManager(Game game) {
        this.game = game;
    }

    //Check Obstacle always before tool
    public void update(Physics physics, GameScene scene) {
        this.player = scene.getPlayer();
        PlayerCollisionData data = physics.getPlayerCollisionData(scene);
        if (data.isTerrainCollision())
            checkTerrainCollision(data.getTerrainCollisionData());
        if (data.isObstacleCollision())
            checkObstacleCollision(data.getObstacleCollisionData());
        if (data.isToolCollision())
            checkToolCollision(data.getToolCollisionData());
        if (data.isStarCollision())
            game.onStarCollision((Star) data.getStarCollisionData().getEntity());
    }

    private void checkTerrainCollision(EntityCollisionData data) {
        checkEntityCollision(data, -0.01f);
        if (data.getEdge() != Edge.TOP)
            game.onGameOver(true);
    }

    private void checkToolCollision(ToolCollisionData data) {
        for (Tool tool : data.getTools())
            tool.onPlayerCollision(player);
    }

    private void checkObstacleCollision(ObstacleCollisionData data) {
        for (EntityCollisionData entityData : data.getCollisions()) {
            checkEntityCollision(entityData, 0);
            if (entityData.getEdge() != Edge.TOP)
                game.onGameOver(true);
        }
    }

    private void checkEntityCollision(EntityCollisionData data, float xBounceFactor) {
        switch (data.getEdge()) {
            case TOP:
                player.getMovement().y = 0;
                player.setToTerrain(data.getEntity().getTopEdge());
                break;
            case LEFT:
                player.getMovement().x *= xBounceFactor;
                player.getPosition().x = data.getEntity().getLeftEdge() - player.RADIUS;
                break;
            case RIGHT:
                player.getMovement().x *= xBounceFactor;
                player.getPosition().x = data.getEntity().getRightEdge() + player.RADIUS;
                break;
        }
    }

}
