package com.example.julian.endlessroll.entities.tools;

import com.example.julian.endlessroll.data.Vector;
import com.example.julian.endlessroll.entities.Animation;
import com.example.julian.endlessroll.entities.Entity;
import com.example.julian.endlessroll.entities.Player;
import com.example.julian.endlessroll.entities.collision.geometry.Geometry;
import com.example.julian.endlessroll.main.game.Timer;

/**
 * Created by Julian on 04.01.2016.
 */
public abstract class Tool extends Entity {

    private boolean placedByRightEdge;
    private boolean updateBounds;
    private Geometry collisionBounds;
    protected Animation animation;
    private boolean floating = false;

    public Tool(ToolType type, Vector position, float width, float height, boolean updateBounds, boolean placedByRightEdge) {
        super(type.getToolTexture(), position, width, height);
        this.updateBounds = updateBounds;
        this.placedByRightEdge = placedByRightEdge;
        animation = new Animation();
        collisionBounds = createCollisionBounds();
    }

    public void update(Timer timer) {
        animation.update(timer);
        super.getTexture().setAtlasIndex(animation.getCurrentTexIndex());
    }

    public abstract void onPlayerCollision(Player player);

    protected abstract Geometry createCollisionBounds();

    public Geometry getCollisionBounds() {
        if (updateBounds)
            return createCollisionBounds();
        return collisionBounds;
    }

    public void setFloating(boolean floating) {
        this.floating = floating;
    }

    public boolean isFloating() {
        return floating;
    }

    public boolean isPlacedByRightEdge() {
        return placedByRightEdge;
    }
}
