package com.example.julian.endlessroll.entities.tools;

import android.support.annotation.Nullable;

import com.example.julian.endlessroll.R;
import com.example.julian.endlessroll.data.Vector;
import com.example.julian.endlessroll.entities.particles.ParticleSystem;
import com.example.julian.endlessroll.entities.textures.Texture;
import com.example.julian.endlessroll.entities.textures.TexturePack;
import com.example.julian.endlessroll.sounds.SoundManager;

public enum ToolType {

    //Check newInstance when new Tool is added!
    RAMP("Ramp", R.drawable.ramp, R.drawable.rampbutton, R.raw.ramp, 3f, 0),
    SPRING("Spring", R.drawable.spring, R.drawable.springbutton, R.raw.ramp, 4f, 5),
    MAGNET("Magnet", R.drawable.magnet, R.drawable.magnetbutton, R.raw.ramp, 6f, 10),
    BOMB("Bomb", R.drawable.bomb, R.drawable.bombbutton, R.raw.ramp, 6f, 12),
    NONE("None", -1, R.drawable.emptybutton, -1, 0, 0);

    private String name;
    private int toolTextureId;
    private Texture toolTexture = null;
    private int buttonDrawable;
    private int placingSoundId;
    private int placingSound = -1;
    private float regenerationTime;
    private int costs;
    private boolean bought = false;

    ToolType(String name, int toolTextureId, int buttonDrawable, int placingSoundId, float regenerationTime, int costs) {
        this.name = name;
        this.toolTextureId = toolTextureId;
        this.buttonDrawable = buttonDrawable;
        this.placingSoundId = placingSoundId;
        this.regenerationTime = regenerationTime;
        this.costs = costs;
    }

    @Nullable
    public Tool newInstance(Vector position, ParticleSystem particleSystem) {
        Tool tool = null;
        switch (this) {
            case RAMP:
                tool = new Ramp(position);
                break;
            case SPRING:
                tool = new Spring(position);
                break;
            case MAGNET:
                tool = new Magnet(position, particleSystem);
                break;
            case BOMB:
                tool = new Bomb(position);
                break;
            case NONE:
                break;
        }
        if (tool != null && tool.isPlacedByRightEdge())
            tool.move(new Vector(-tool.getWidth() / 2, 0));
        return tool;
    }

    public void loadAllPlacingSounds(SoundManager soundManager) {
        for (ToolType type : values())
            type.loadPlacingSound(soundManager);
    }

    public static void loadAllToolTextures(TexturePack texturePack) {
        for (ToolType type : values())
            type.loadToolTexture(texturePack);
    }

    private void loadPlacingSound(SoundManager soundManager) {
        if (placingSoundId == -1)
            return;
        placingSound = soundManager.loadSound(placingSoundId);
    }

    private void loadToolTexture(TexturePack texturePack) {
        if (toolTextureId == -1)
            return;
        toolTexture = texturePack.loadAtlas(toolTextureId, 2, 2);
    }

    public Texture getToolTexture() {
        return toolTexture;
    }

    public int getButtonDrawable() {
        return buttonDrawable;
    }

    public int getPlacingSound() {
        return placingSound;
    }

    public String getName() {
        return name;
    }

    public void setBought(boolean bought) {
        this.bought = bought;
    }

    public boolean isBought() {
        return bought;
    }

    public int getCosts() {
        return costs;
    }

    public float getRegenerationTime() {
        return regenerationTime;
    }
}