package com.example.julian.endlessroll.levels;

import android.content.Context;

import com.example.julian.endlessroll.main.DataStorageHandler;
import com.example.julian.endlessroll.main.GameLog;

import org.simpleframework.xml.Serializer;
import org.simpleframework.xml.core.Persister;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

public class LevelManager extends ArrayList<LevelPack> {

    public LevelManager(Context context, DataStorageHandler dataStorageHandler) throws Exception {
        String[] assets = context.getAssets().list("levelpacks");
        for (String asset : assets) {
            try {
                LevelPack pack = loadLevelPack(context, "levelpacks/" + asset);
                dataStorageHandler.readLevelProgress(pack);
                dataStorageHandler.readLevelPackLocked(pack);
                pack.tryToUnlockFirstLevel();
                if (pack.getId() == 0)
                    pack.setLocked(false);
                super.add(pack);
            } catch (Exception e) {
                GameLog.e(e);
            }
            sort();
        }
    }

    private void sort() {
        Collections.sort(this, new Comparator<LevelPack>() {
            @Override
            public int compare(LevelPack lhs, LevelPack rhs) {
                return lhs.getId() - rhs.getId();
            }
        });
    }

    private LevelPack loadLevelPack(Context context, String name) throws Exception {
        try {
            InputStream source = context.getAssets().open(name);
            Serializer serializer = new Persister();
            return serializer.read(LevelPack.class, source);
        } catch (Exception e) {
            throw new Exception("Could not load levelPack \"" + name + "\"", e);
        }
    }

    public void reset() {
        for (LevelPack pack : this) {
            pack.reset();
            if (pack.getId() != 0)
                pack.setLocked(true);
        }
    }

    public void unlockAllPacks() {
        for (LevelPack levelPack : this)
            levelPack.setLocked(false);
    }

    public void unlockAllLevels() {
        for (LevelPack levelPack : this) {
            for (Level level : levelPack.getLevels())
                level.setUnlocked(true);
        }
    }

    public LevelPack nextLevelPack(LevelPack old) {
        int oldId = old.getId();
        for (LevelPack pack : this) {
            if (pack.getId() == oldId + 1)
                return pack;
        }
        return null;
    }

}
