package com.example.julian.endlessroll.levels.levelup;

import android.content.Context;

import com.example.julian.endlessroll.entities.tools.ToolType;
import com.example.julian.endlessroll.views.BountyMessage;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Julian on 17.07.2016.
 */
public class LevelUpBounties extends HashMap<Integer, List<LevelUpBounties.Bounty>> {

    public interface Bounty {
        BountyMessage createMessage(Context context);
    }

    private Map<ToolBounty, Integer> allToolBounties = new HashMap<>();
    private List<Integer> allToolSlotLevels = new ArrayList<>();

    public LevelUpBounties() {
        addBounties(1, new ToolBounty(ToolType.RAMP), new ToolSlotBounty());
        addBounties(2, new StarBounty(5), new ToolBounty(ToolType.SPRING));
        addBounties(3, new StarBounty(6), new ToolBounty(ToolType.MAGNET), new ToolSlotBounty());
        addBounties(4, new StarBounty(7), new ToolBounty(ToolType.BOMB), new ToolSlotBounty());
        addBounties(5, new StarBounty(8), new ToolSlotBounty());
    }

    private void addBounties(int unlockLevel, Bounty... bounties) {
        List<Bounty> list = new ArrayList<>();
        for (Bounty bounty : bounties) {
            list.add(bounty);
            if (bounty instanceof ToolBounty)
                allToolBounties.put((ToolBounty) bounty, unlockLevel);
            else if (bounty instanceof ToolSlotBounty)
                allToolSlotLevels.add(unlockLevel);
        }
        super.put(unlockLevel, list);
    }

    public boolean isToolAlreadyUnlockedAtLevel(int level, ToolType toolType) {
        for (ToolBounty bounty : allToolBounties.keySet())
            if (bounty.getToolType() == toolType && allToolBounties.get(bounty) <= level)
                return true;
        return false;
    }

    public boolean isToolSlotUnlockedAtLevel(int toolSlotIndex, int level) {
        return allToolSlotLevels.get(toolSlotIndex) <= level;
    }

    public int getBountyStarsAtLevel(int level) {
        for (Bounty bounty : this.get(level)) {
            if (bounty instanceof StarBounty)
                return ((StarBounty) bounty).getStarCount();
        }
        return 0;
    }

    @Override
    public List<Bounty> get(Object key) {
        List<Bounty> list = super.get(key);
        if (list == null)
            return new ArrayList<>();
        return list;
    }
}
