package com.example.julian.endlessroll.main;

import android.app.Activity;
import android.content.Context;
import android.content.SharedPreferences;

import com.example.julian.endlessroll.entities.tools.ToolSlot;
import com.example.julian.endlessroll.entities.tools.ToolSlotSettings;
import com.example.julian.endlessroll.levels.Level;
import com.example.julian.endlessroll.levels.LevelPack;
import com.example.julian.endlessroll.main.game.User;
import com.example.julian.endlessroll.sqlDatabase.MyDatabase;

/**
 * Created by Julian on 25.04.2016.
 */
public class DataStorageHandler {

    private final String PREFERENCES_NAME = "GamePreferences";
    private final String SOUND_ON = "Sound";
    private final String USER_STARS = "Stars";
    private final String USER_LEVEL = "Level";
    private final String USER_LEVEL_PROGRESS = "LevelProgress";

    private SharedPreferences preferences;
    private MyDatabase database;

    public DataStorageHandler(Activity activity) {
        preferences = activity.getSharedPreferences(PREFERENCES_NAME, Context.MODE_PRIVATE);
        database = new MyDatabase(activity);
    }

    public boolean readIsSoundOn() {
        return preferences.getBoolean(SOUND_ON, true);
    }

    public void writeSoundOn(boolean soundOn) {
        SharedPreferences.Editor editor = preferences.edit();
        editor.putBoolean(SOUND_ON, soundOn);
        editor.apply();
    }

    public User readUserData() {
        int stars = preferences.getInt(USER_STARS, 0);
        int level = preferences.getInt(USER_LEVEL, 1);
        int levelProgress = preferences.getInt(USER_LEVEL_PROGRESS, 0);
        //TODO: Default Tooltypes to NONE
        database.open();
        ToolSlotSettings toolSlotSettings = database.readToolSlotSettings();
        database.close();
        return new User(stars, level, levelProgress, toolSlotSettings);
    }

    public void writeUserData(User user) {
        SharedPreferences.Editor editor = preferences.edit();
        editor.putInt(USER_STARS, user.getStarCount());
        editor.putInt(USER_LEVEL, user.getLevel());
        editor.putInt(USER_LEVEL_PROGRESS, user.getLevelProgress());
        editor.apply();
        database.open();
        for (ToolSlot toolSlot : user.getToolSlotSettings())
            database.writeToolSlot(user.getToolSlotSettings().indexOf(toolSlot), toolSlot);
        database.close();
    }

    public void writeLevelProgress(int levelpackNumber, Level level) {
        database.open();
        database.writeLevelProgress(levelpackNumber, level);
        database.close();
    }

    public void writeLevelPackLocked(LevelPack levelPack) {
        database.open();
        database.writeLevelPackLocked(levelPack);
        database.close();
    }

    public void readLevelPackLocked(LevelPack levelPack){
        database.open();
        database.readLevelPackLocked(levelPack);
        database.close();
    }

    public void readLevelProgress(LevelPack levelPack) {
        database.open();
        for (Level level : levelPack.getLevels()) {
            database.readLevelProgress(levelPack.getId(), level);
        }
        database.close();
    }

    public void clearLevelProgess() {
        database.open();
        database.clearLevelProgess();
        database.close();
    }

    public void readBoughtTools() {
        database.open();
        database.readBoughtTools();
        database.close();
    }

    public void writeBoughtTools() {
        database.open();
        database.writeBoughtTools();
        database.close();
    }

}
