package com.example.julian.endlessroll.main.game;

import com.example.julian.endlessroll.data.Vector;
import com.example.julian.endlessroll.entities.Background;
import com.example.julian.endlessroll.entities.Entity;
import com.example.julian.endlessroll.entities.Player;
import com.example.julian.endlessroll.entities.Terrain;
import com.example.julian.endlessroll.entities.particles.ParticleSystem;
import com.example.julian.endlessroll.entities.textures.TexturePack;
import com.example.julian.endlessroll.rendering.Lock;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * Created by Julian on 20.07.2016.
 */
public abstract class Scene extends ArrayList<Entity> {

    public float cameraX;
    public final Lock lock;

    private Vector screenSize;
    private Entity playerArrow;

    protected ParticleSystem particleSystem;
    protected TexturePack textures;
    protected Background background;
    protected Terrain terrain;
    protected Player player;

    public Scene(TexturePack texturePack, ParticleSystem particleSystem) {
        this.particleSystem = particleSystem;
        setTexturePack(texturePack);
        playerArrow = new Entity(textures.playerArrow, new Vector(0, 0.9f), .2f, .2f);
        background = new Background(textures.background);
        terrain = new Terrain(textures.terrain);
        player = new Player(textures.player);
        lock = new Lock();
    }

    public void setTexturePack(TexturePack texturePack) {
        this.textures = texturePack;
    }

    public void update(Timer timer) {
        lock.lock();
        Iterator<Entity> iterator = super.iterator();
        while (iterator.hasNext()) {
            Entity entity = iterator.next();
            Vector movement = entity.getMovement();
            Vector finalMovement = new Vector(movement).mul(timer.getFrameTime());
            entity.move(finalMovement);
            if (entity.isDestroyed() && entity.getDestroyEffect() != null)
                entity.getDestroyEffect().createEffect(particleSystem, new Vector(entity.getPosition()), new Vector(entity.getWidth(), entity.getHeight())).start();
            if (entity.equals(player))
                moveEnviroment(finalMovement.x);
            else if (entity.getRightEdge() - cameraX < -3f || entity.isDestroyed()) {
                iterator.remove();
                removeEntityFromAllLists(entity);
            }
        }

        if (player.getPosition().y >= player.RADIUS + 1) {
            playerArrow.getPosition().x = player.getPosition().x;
            if (!super.contains(playerArrow)) {
                super.add(playerArrow);
            }
        } else super.remove(playerArrow);
        lock.unlock();
    }

    @Override
    public boolean remove(Object object) {
        removeEntityFromAllLists((Entity) object);
        return super.remove(object);
    }

    protected abstract void removeEntityFromAllLists(Entity entity);

    private void moveEnviroment(float x) {
        cameraX += x;
        background.move(x * 0.95f, cameraX);
        terrain.update(cameraX);
    }

    protected Vector calcWorldFromScreenCoords(float screenX, float screenY) throws Exception {
        if (screenSize == null)
            throw new Exception("ScreenSize not set");
        float glCoordWidth = (2f * screenSize.x / screenSize.y);
        float x = ((screenX / screenSize.x) * 2f - 1f) * glCoordWidth / 2;
        x += cameraX;
        float y = -((screenY / screenSize.y) * 2f - 1f);
        return new Vector(x, y);
    }

    public void setScreenSize(Vector screenSize) {
        this.screenSize = screenSize;
    }

    public Background getBackground() {
        return background;
    }

    public Terrain getTerrain() {
        return terrain;
    }

    public Player getPlayer() {
        return player;
    }

    public TexturePack getTextures() {
        return textures;
    }

    public ParticleSystem getParticleSystem() {
        return particleSystem;
    }
}
