package com.example.julian.endlessroll.main.screens;

import android.widget.RelativeLayout;

import com.example.julian.endlessroll.R;
import com.example.julian.endlessroll.levels.Level;
import com.example.julian.endlessroll.levels.LevelPack;
import com.example.julian.endlessroll.main.GameActivity;
import com.example.julian.endlessroll.main.GameHandler;
import com.example.julian.endlessroll.main.MyGlSurfaceView;
import com.example.julian.endlessroll.main.game.Game;
import com.example.julian.endlessroll.main.game.GameState;
import com.example.julian.endlessroll.views.TopBarData;

/**
 * Created by Julian on 08.02.2016.
 */
public class GameScreen extends GLScreen<RelativeLayout> {

    private GameActivity activity;
    private Game game;

    public GameScreen(TopBarData topBarData, MyGlSurfaceView glSurfaceView) throws Exception{
        super(ScreenType.GAME, topBarData.getGameActivity(), R.layout.game, glSurfaceView);
        this.activity = topBarData.getGameActivity();
        game = new Game(gameViewHandler, topBarData);
        glView.addRendering(game);
    }

    @Override
    public void prepareToBeShown() {
        glView.setCurrentRendering(game);
        game.resetViews();
    }

    public void onPause(){
       game.pause();
    }

    @Override
    public void onBackKeyDown() {
        if (game.getGameState() == GameState.RUNNING)
            game.pause();
    }

    public void startGame(LevelPack levelPack, Level level) {
        game.startGame(levelPack, level);
    }

    private GameHandler gameViewHandler = new GameHandler() {

        @Override
        public void startInUiThread(Runnable runnable) {
            activity.runOnUiThread(runnable);
        }

        @Override
        public void toLevelsScreen() {
            glView.setCurrentRendering(null);
            activity.flipToScreen(ScreenType.LEVELS);
        }

        @Override
        public RelativeLayout getRootLayout() {
            return layout;
        }

        @Override
        public void onException(Exception e) {
            activity.onException(e);
        }
    };

}
