package com.example.julian.endlessroll.main.screens;

import android.view.DragEvent;
import android.view.MotionEvent;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.FrameLayout;
import android.widget.HorizontalScrollView;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;

import com.example.julian.endlessroll.R;
import com.example.julian.endlessroll.entities.tools.ToolSlotSettings;
import com.example.julian.endlessroll.entities.tools.ToolType;
import com.example.julian.endlessroll.levels.levelup.LevelUpBounties;
import com.example.julian.endlessroll.main.DataStorageHandler;
import com.example.julian.endlessroll.main.GameActivity;
import com.example.julian.endlessroll.main.game.User;
import com.example.julian.endlessroll.views.ToolOfferSlot;
import com.example.julian.endlessroll.views.TopBar;
import com.example.julian.endlessroll.views.TopBarData;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Julian on 08.07.2016.
 */
public class ToolShopScreen extends Screen<RelativeLayout> implements View.OnDragListener, View.OnClickListener, View.OnTouchListener {

    private LevelUpBounties levelUpBounties;
    private GameActivity activity;
    private ScreenType caller;
    private ToolSlotSettings slotSettings;
    private User user;
    private DataStorageHandler dataStorageHandler;
    private ToolOfferSlot selectedToolOfferSlot;

    private TopBar topBar;
    private Button priceButton;
    private ImageView toolSlot1;
    private ImageView toolSlot2;
    private ImageView toolSlot3;
    private ImageView toolSlot4;
    private HorizontalScrollView scrollView;
    private LinearLayout toolOfferLayout;
    private List<ToolOfferSlot> toolOfferSlots = new ArrayList<>();

    public ToolShopScreen(TopBarData topBarData) {
        super(ScreenType.TOOL_SHOP, topBarData.getGameActivity(), R.layout.toolshop);
        this.levelUpBounties = new LevelUpBounties();
        this.activity = topBarData.getGameActivity();
        this.user = topBarData.getUser();
        this.slotSettings = user.getToolSlotSettings();
        this.dataStorageHandler = topBarData.getDataStorageHandler();
        layout.setOnClickListener(this);
        topBar = new TopBar(topBarData, ScreenType.TOOL_SHOP, (RelativeLayout) layout.findViewById(R.id.toolshop_topbar));
        priceButton = (Button) layout.findViewById(R.id.toolshop_price_button);
        priceButton.setOnClickListener(this);
        priceButton.setTypeface(topBarData.getTypeface());
        toolSlot1 = getToolSlotView(R.id.toolshop_slot1);
        toolSlot2 = getToolSlotView(R.id.toolshop_slot2);
        toolSlot3 = getToolSlotView(R.id.toolshop_slot3);
        toolSlot4 = getToolSlotView(R.id.toolshop_slot4);
        scrollView = (HorizontalScrollView) layout.findViewById(R.id.toolshop_scrollview);
        scrollView.setOnTouchListener(this);
        toolOfferLayout = (LinearLayout) layout.findViewById(R.id.toolshop_tool_offer_layout);
        LinearLayout.LayoutParams params = new LinearLayout.LayoutParams(ViewGroup.LayoutParams.WRAP_CONTENT, ViewGroup.LayoutParams.WRAP_CONTENT);
        params.setMargins(20, 0, 20, 0);
        for (ToolType type : ToolType.values()) {
            if (type != ToolType.NONE) {
                ToolOfferSlot slot = new ToolOfferSlot(this, activity, topBarData.getTypeface(), type, levelUpBounties);
                toolOfferSlots.add(slot);
                toolOfferLayout.addView(slot.getLayout(), params);
            }
        }
    }

    public void showPriceButton(ToolOfferSlot slot) {
        selectedToolOfferSlot = slot;
        int[] slotLocation = new int[2];
        slot.getImage().getLocationOnScreen(slotLocation);
        priceButton.setX(slotLocation[0]);
        priceButton.setY(slotLocation[1] + slot.getImage().getHeight() * 0.75f);
        int costs = slot.getToolType().getCosts();
        priceButton.setText(costs + "");
        priceButton.setEnabled(costs <= user.getStarCount());
        priceButton.setVisibility(View.VISIBLE);
    }

    private ImageView getToolSlotView(int id) {
        FrameLayout slotLayout = (FrameLayout) layout.findViewById(id);
        ImageView imageView = (ImageView) slotLayout.findViewById(R.id.toolslot_image);
        imageView.setOnDragListener(this);
        return imageView;
    }

    public void setCaller(ScreenType caller) {
        this.caller = caller;
    }

    @Override
    public void prepareToBeShown() {
        topBar.update();
        slotSettings.unlockSlotsIfLevelReached(user.getLevel(), levelUpBounties);
        toolSlot1.setImageResource(slotSettings.get(0).getDrawable());
        toolSlot2.setImageResource(slotSettings.get(1).getDrawable());
        toolSlot3.setImageResource(slotSettings.get(2).getDrawable());
        toolSlot4.setImageResource(slotSettings.get(3).getDrawable());
        for (ToolOfferSlot slot : toolOfferSlots) {
            slot.updateIsBought();
            boolean locked = !levelUpBounties.isToolAlreadyUnlockedAtLevel(user.getLevel(), slot.getToolType());
            slot.setLocked(locked);
            if (slot.getToolType() == ToolType.RAMP)
                slot.buy();
        }
    }

    @Override
    public void onBackKeyDown() {
        priceButton.setVisibility(View.INVISIBLE);
        activity.flipToScreen(caller);
    }

    @Override
    public boolean onDrag(View v, DragEvent event) {
        if (event.getAction() == DragEvent.ACTION_DRAG_ENDED) {
            for (ToolOfferSlot toolOfferSlot : toolOfferSlots)
                toolOfferSlot.setDragged(false);
            return true;
        }
        int slot = -1;
        slot = v.equals(toolSlot1) ? 0 : slot;
        slot = v.equals(toolSlot2) ? 1 : slot;
        slot = v.equals(toolSlot3) ? 2 : slot;
        slot = v.equals(toolSlot4) ? 3 : slot;
        if (!slotSettings.get(slot).isLocked()) {
            if (event.getAction() == DragEvent.ACTION_DRAG_ENTERED) {
                //TODO: animation
            }
            if (event.getAction() == DragEvent.ACTION_DRAG_EXITED) {
                //TODO: ggf. animation
            }
            if (event.getAction() == DragEvent.ACTION_DROP) {
                //TODO: animation??
                    slotSettings.changeToolSlotType(slot, getDraggedToolType());
                    prepareToBeShown();
                    dataStorageHandler.writeUserData(user);
            }
        }

        return true;
    }

    private ToolType getDraggedToolType() {
        for (ToolOfferSlot toolOfferSlot : toolOfferSlots) {
            if (toolOfferSlot.isDragged())
                return toolOfferSlot.getToolType();
        }
        return ToolType.NONE;
    }

    @Override
    public void onClick(View v) {
        if (v.equals(layout)) {
            priceButton.setVisibility(View.INVISIBLE);
        } else if (v.equals(priceButton)) {
            int costs = selectedToolOfferSlot.getToolType().getCosts();
            topBar.showStarcountDecrease(-costs);
            user.decreaseStarCount(costs);
            selectedToolOfferSlot.buy();
            priceButton.setVisibility(View.INVISIBLE);
            topBar.update();
            dataStorageHandler.writeBoughtTools();
        }
    }

    @Override
    public boolean onTouch(View v, MotionEvent event) {
        priceButton.setVisibility(View.INVISIBLE);
        return false;
    }
}
