package com.example.julian.endlessroll.rendering.shader;

import android.content.Context;
import android.opengl.GLES20;

import com.example.julian.endlessroll.data.Color;
import com.example.julian.endlessroll.entities.Entity;
import com.example.julian.endlessroll.rendering.MatrixCreator;

/**
 * Created by Julian on 10.08.2016.
 */
public class ParticleShader extends ShaderProgram{

    private int location_mvpMatrix;
    private int location_transformationMatrix;
    private int location_color;

    public ParticleShader(Context context) throws Exception {
        super(context, "shader/entityVertexShader.glsl", "shader/particleFragmentShader.glsl");
    }

    @Override
    protected void loadUniformLocations() {
        location_mvpMatrix = super.getUniformLocation("mvpMatrix");
        location_transformationMatrix = super.getUniformLocation("transformationMatrix");
        location_color = super.getUniformLocation("color");
    }

    public void loadMVPMatrix(MatrixCreator matrixCreator, float cameraX) {
        float[] mvpMatrix = matrixCreator.createModelViewProjectionMatrix(cameraX);
        GLES20.glUniformMatrix4fv(location_mvpMatrix, 1, false, mvpMatrix, 0);
    }

    public void loadTransformationMatrix(MatrixCreator matrixCreator, Entity entity) {
        float[] transformationMatrix = matrixCreator.createTransformationMatrix(entity);
        GLES20.glUniformMatrix4fv(location_transformationMatrix, 1, false, transformationMatrix, 0);
    }

    public void loadColor(Color color) {
        GLES20.glUniform3f(location_color, color.getR(), color.getG(), color.getB());
    }

}
