package com.example.julian.endlessroll.sqlDatabase;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;

import com.example.julian.endlessroll.entities.tools.ToolSlot;
import com.example.julian.endlessroll.entities.tools.ToolSlotSettings;
import com.example.julian.endlessroll.entities.tools.ToolType;
import com.example.julian.endlessroll.levels.Level;
import com.example.julian.endlessroll.levels.LevelPack;
import com.example.julian.endlessroll.main.GameLog;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by Julian on 10.05.2016.
 */
public class MyDatabase extends DatabaseAdapter {

    private final static int VERSION = 5;
    private final static String DATABASE_NAME = "DATABASE";
    private SQLTableColumn levelpackColumn, levelColumn, unlockedColumn, completedColumn, starsColumn;
    private SQLTableColumn idColumn, toolTypeColumn, lockedColumn, boughtColumn;
    private SQLTable levelTable, levelPackTable, toolSlotTable, boughtToolTable;

    public MyDatabase(Context context) {
        super(context, DATABASE_NAME, VERSION);
    }

    @Override
    protected void createColumns() {
        //TODO: no names needed for columns and tables
        //TODO: really use database adapter???
        levelpackColumn = new SQLTableColumn("LEVELPACK", SQLTableColumn.ColumnType.INTEGER, SQLTableColumn.ColumnExtra.NOT_NULL);
        levelColumn = new SQLTableColumn("LEVEL", SQLTableColumn.ColumnType.INTEGER, SQLTableColumn.ColumnExtra.NOT_NULL);
        unlockedColumn = new SQLTableColumn("UNLOCKED", SQLTableColumn.ColumnType.INTEGER, SQLTableColumn.ColumnExtra.NOT_NULL);
        completedColumn = new SQLTableColumn("COMPLETED", SQLTableColumn.ColumnType.INTEGER, SQLTableColumn.ColumnExtra.NOT_NULL);
        starsColumn = new SQLTableColumn("STARS", SQLTableColumn.ColumnType.TEXT, SQLTableColumn.ColumnExtra.NONE);

        idColumn = new SQLTablePrimaryKeyColumn(false);
        toolTypeColumn = new SQLTableColumn("TOOLTYPE", SQLTableColumn.ColumnType.INTEGER, SQLTableColumn.ColumnExtra.NOT_NULL);
        lockedColumn = new SQLTableColumn("LOCKED", SQLTableColumn.ColumnType.INTEGER, SQLTableColumn.ColumnExtra.NOT_NULL);

        boughtColumn = new SQLTableColumn("BOUGHT", SQLTableColumn.ColumnType.INTEGER, SQLTableColumn.ColumnExtra.NOT_NULL);
    }

    @Override
    protected SQLTable[] createTables() {
        levelTable = new SQLTable("LEVELTABLE", levelpackColumn, levelColumn, unlockedColumn, completedColumn, starsColumn);
        levelPackTable = new SQLTable("LEVELPACKS", levelpackColumn, unlockedColumn);
        toolSlotTable = new SQLTable("TOOL_SLOT_TABLE", idColumn, toolTypeColumn, lockedColumn);
        boughtToolTable = new SQLTable("BOUGHT_TOOL_TABLE", idColumn, boughtColumn);
        return new SQLTable[]{levelTable, levelPackTable, toolSlotTable, boughtToolTable};
    }

    public void writeLevelProgress(int levelpackNumber, Level level) {
        ContentValues values = new ContentValues();
        values.put(levelpackColumn.getKey(), levelpackNumber);
        values.put(levelColumn.getKey(), level.getNumber());
        values.put(unlockedColumn.getKey(), level.isUnlocked() ? 1 : 0);
        values.put(completedColumn.getKey(), level.isCompleted() ? 1 : 0);
        values.put(starsColumn.getKey(), level.getCollectedStars());
        long update = super.update(levelTable, values, levelpackColumn, levelpackNumber, levelColumn, level.getNumber());

        readLevelProgress(levelpackNumber, level);
    }

    public void readLevelProgress(int levelpackNumber, Level level) {
        Cursor cursor = super.getCursor(levelTable, levelpackColumn, levelpackNumber, levelColumn, level.getNumber());
        if (cursor.moveToFirst()) {
            boolean unlocked = cursor.getInt(2) == 1;
            boolean completed = cursor.getInt(3) == 1;
            String stars = cursor.getString(4);
            level.setUnlocked(unlocked);
            level.setCompleted(completed);
            level.setCollectedStars(stars);
        }
        cursor.close();
    }

    public void clearLevelProgess() {
        super.clearTable(levelTable);
    }

    public void writeLevelPackLocked(LevelPack levelPack) {
        ContentValues values = new ContentValues();
        values.put(levelpackColumn.getKey(), levelPack.getId());
        values.put(unlockedColumn.getKey(), levelPack.isLocked() ? 0 : 1);
        super.update(levelPackTable, values, levelpackColumn, levelPack.getId());
    }

    public void readLevelPackLocked(LevelPack levelPack) {
        Cursor cursor = super.getCursor(levelPackTable, levelpackColumn, levelPack.getId());
        if (cursor.moveToFirst()) {
            boolean unlocked = cursor.getInt(1) == 1;
            levelPack.setLocked(!unlocked);
        }else{
            levelPack.setLocked(true);
        }
        cursor.close();
    }

    public void writeToolSlot(int id, ToolSlot toolSlot) {
        ContentValues values = new ContentValues();
        values.put(idColumn.getKey(), id);
        values.put(toolTypeColumn.getKey(), toolSlot.getToolType().ordinal());
        values.put(lockedColumn.getKey(), toolSlot.isLocked() ? 1 : 0);
        super.update(toolSlotTable, values, idColumn, id);
    }

    public ToolSlotSettings readToolSlotSettings() {
        List<ToolSlot> list = new ArrayList<>();
        Cursor cursor = super.getCursor(toolSlotTable);
        if (cursor.moveToFirst()) {
            do {
                ToolType toolType = ToolType.values()[cursor.getInt(1)];
                boolean locked = cursor.getInt(2) == 1;
                list.add(new ToolSlot(toolType, locked));
            }
            while (cursor.moveToNext());
        }
        cursor.close();
        GameLog.i("read toolSlotList: " + list.size());
        if (list.isEmpty())
            return new ToolSlotSettings();
        else
            return new ToolSlotSettings(list);
    }

    public void writeBoughtTools() {
        for (ToolType tool : ToolType.values()) {
            if (tool != ToolType.NONE) {
                ContentValues values = new ContentValues();
                values.put(idColumn.getKey(), tool.ordinal());
                values.put(boughtColumn.getKey(), tool.isBought() ? 1 : 0);
                super.update(boughtToolTable, values, idColumn, tool.ordinal());
            }
        }
    }

    public void readBoughtTools() {
        Cursor cursor = super.getCursor(boughtToolTable);
        if (cursor.moveToFirst()) {
            do {
                int id = cursor.getInt(0);
                boolean bought = cursor.getInt(1) == 1;
                ToolType.values()[id].setBought(bought);
            }
            while (cursor.moveToNext());
        }
        cursor.close();
    }

}
