package com.example.julian.endlessroll.views;

import android.content.ClipData;
import android.content.Context;
import android.graphics.Typeface;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.GradientDrawable;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.example.julian.endlessroll.R;
import com.example.julian.endlessroll.entities.tools.ToolType;
import com.example.julian.endlessroll.levels.levelup.LevelUpBounties;
import com.example.julian.endlessroll.main.screens.ToolShopScreen;

/**
 * Created by Julian on 16.07.2016.
 */
public class ToolOfferSlot implements View.OnClickListener, View.OnLongClickListener {

    private ToolShopScreen toolShopScreen;
    private boolean dragged;
    private Context context;
    private ToolType toolType;
    private LevelUpBounties levelUpBounties;
    private boolean locked;
    private boolean bought;

    private LinearLayout layout;
    private TextView title;
    private ImageView image;

    public ToolOfferSlot(ToolShopScreen toolShopScreen, Context context, Typeface typeface, ToolType toolType, LevelUpBounties levelUpBounties) {
        this.toolShopScreen = toolShopScreen;
        this.context = context;
        this.toolType = toolType;
        this.levelUpBounties = levelUpBounties;
        bought = toolType.isBought();
        LayoutInflater inflater = LayoutInflater.from(context);
        layout = (LinearLayout) inflater.inflate(R.layout.tool_offer_slot, null);
        layout.setOnClickListener(this);
        layout.setOnLongClickListener(this);
        FrameLayout slotLayout = (FrameLayout) layout.findViewById(R.id.toolofferslot_slot);
        title = (TextView) layout.findViewById(R.id.toolofferslot_title);
        title.setTypeface(typeface);
        image = (ImageView) slotLayout.findViewById(R.id.toolslot_image);
        image.setBackgroundDrawable(createColoredBackground());
        setLocked(false);
    }

    public void setLocked(boolean locked) {
        this.locked = locked;
        if (locked) {
            title.setText("Level up to unlock");
            image.setImageResource(R.drawable.lockedbutton);
        } else {
            title.setText(toolType.getName());
            image.setImageResource(toolType.getButtonDrawable());
        }
    }

    public void updateIsBought(){
        bought = toolType.isBought();
        image.setBackgroundDrawable(createColoredBackground());
    }

    public void buy() {
        this.bought = true;
        toolType.setBought(true);
        setColor(R.color.toolslotEnabled);
    }

    private Drawable createColoredBackground() {
        GradientDrawable gd = new GradientDrawable();
        if (bought) {
            gd.setColor(context.getResources().getColor(R.color.toolslotEnabled));
        } else {
            gd.setColor(context.getResources().getColor(R.color.toolslotDisabled));
        }
        gd.setCornerRadius(10);
        return gd;
    }

    private void setColor(int id) {
        GradientDrawable gd = (GradientDrawable) image.getBackground();
        gd.setColor(context.getResources().getColor(id));
    }

    public boolean isLocked() {
        return locked;
    }

    public ToolType getToolType() {
        return toolType;
    }

    public LinearLayout getLayout() {
        return layout;
    }

    public ImageView getImage(){
        return image;
    }

    @Override
    public void onClick(View v) {
        if (!locked && !bought) {
            toolShopScreen.showPriceButton(this);
        }
    }

    @Override
    public boolean onLongClick(View v) {
        if (!locked && bought) {
            dragged = true;
            ClipData clipData = ClipData.newPlainText("", "");
            View.DragShadowBuilder shadowBuilder = new View.DragShadowBuilder(image);
            image.startDrag(clipData, shadowBuilder, image, 0);
            return true;
        }
        return false;
    }

    public boolean isDragged() {
        return dragged;
    }

    public void setDragged(boolean dragged) {
        this.dragged = dragged;
    }
}
