package com.example.julian.endlessroll.views;

import android.view.View;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;
import android.widget.ToggleButton;

import com.example.julian.endlessroll.R;
import com.example.julian.endlessroll.entities.tools.ToolType;
import com.example.julian.endlessroll.main.screens.Screen;

/**
 * Created by Julian on 08.07.2016.
 */
public class TopBar extends TopBarData implements View.OnClickListener {

    private Animation decreaseAnimation;
//TEST
    private Screen.ScreenType parent;
    private RelativeLayout layout;
    private TextView levelDisplay;
    private ProgressBar levelProgress;
    private TextView starCount;
    private ToggleButton soundToggle;
    private Button toolshopButton;
    private Button resetButton;
    private TextView starCountDecrease;

    public TopBar(TopBarData topBarData, Screen.ScreenType parent, RelativeLayout layout) {
        super(topBarData);
        this.parent = parent;
        this.layout = layout;
        decreaseAnimation = AnimationUtils.loadAnimation(gameActivity, R.anim.decrease);

        levelDisplay = (TextView) layout.findViewById(R.id.topbar_leveldisplay);
        levelDisplay.setTypeface(typeface);
        levelProgress = (ProgressBar) layout.findViewById(R.id.topbar_levelprogress);
        starCount = (TextView) layout.findViewById(R.id.topbar_starcount);
        starCount.setTypeface(typeface);
        soundToggle = (ToggleButton) layout.findViewById(R.id.topbar_soundtoggle);
        soundToggle.setTypeface(typeface);
        soundToggle.setOnClickListener(this);
        toolshopButton = (Button) layout.findViewById(R.id.topbar_toolshop);
        toolshopButton.setTypeface(typeface);
        toolshopButton.setOnClickListener(this);
        if (parent == Screen.ScreenType.TOOL_SHOP || parent == Screen.ScreenType.GAME)
            toolshopButton.setEnabled(false);
        resetButton = (Button) layout.findViewById(R.id.topbar_resetButton);
        resetButton.setTypeface(typeface);
        resetButton.setOnClickListener(this);
        starCountDecrease = (TextView) layout.findViewById(R.id.topbar_starcount_decrease);
        starCountDecrease.setTypeface(typeface);
    }

    public void startAnimation(Animation animation) {
        layout.startAnimation(animation);
    }

    public void update() {
        levelDisplay.setText("Level: " + user.getLevel());
        levelProgress.setProgress(user.getLevelProgress());
        starCount.setText(user.getStarCount() + "");
        soundToggle.setChecked(soundManager.isSoundOn());
    }

    private void flipToToolshop(){
        gameActivity.setToolshopCaller(parent);
        gameActivity.flipToScreen(Screen.ScreenType.TOOL_SHOP);
    }

    public void showStarcountDecrease(int decrease){
        starCountDecrease.setText(decrease+"");
        starCountDecrease.startAnimation(decreaseAnimation);
    }

    @Override
    public void onClick(View v) {
        if (v.equals(soundToggle)) {
            soundManager.setSoundOn(soundToggle.isChecked());
        } else if (v.equals(toolshopButton)) {
            flipToToolshop();
        } else if (v.equals(resetButton)) {
            user.clearData();
            dataStorageHandler.writeUserData(user);
            levelManager.reset();
            dataStorageHandler.clearLevelProgess();
            for(ToolType type : ToolType.values())
                type.setBought(false);
            dataStorageHandler.writeBoughtTools();
        }
    }
}
