package com.example.julian.endlessroll.views;

import android.view.View;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.example.julian.endlessroll.R;
import com.example.julian.endlessroll.data.Vector;
import com.example.julian.endlessroll.main.GameHandler;
import com.example.julian.endlessroll.main.game.Game;
import com.example.julian.endlessroll.main.game.GameState;
import com.example.julian.endlessroll.main.game.Timer;

/**
 * Created by Julian on 11.12.2015.
 */
public class ViewManager implements View.OnClickListener {

    private Vector screenSize;
    private GameHandler gameViewHandler;
    private Game game;

    private TextView fpsView;
    private TextView playerProgress;
    private ImageView pauseButton;
    private Message message;
    public ToolButtonBar toolButtonBar;
    public ShortMenu shortMenu;
    private LevelupMessage levelupMessage;
    private Countdown countdown;

    public ViewManager(final Game game, GameHandler gameViewHandler, final TopBarData topBarData) {
        this.game = game;
        this.gameViewHandler = gameViewHandler;

        final RelativeLayout layout = gameViewHandler.getRootLayout();
        gameViewHandler.startInUiThread(new Runnable() {
            @Override
            public void run() {
                toolButtonBar = new ToolButtonBar(game, topBarData.getUser().getToolSlotSettings(), (LinearLayout) layout.findViewById(R.id.game_toolbuttonbar));
                shortMenu = new ShortMenu(game, topBarData, (FrameLayout) layout.findViewById(R.id.game_shortmenu));
                message = new Message(game, topBarData.getTypeface(), (RelativeLayout) layout.findViewById(R.id.game_message));
                levelupMessage = new LevelupMessage(game.getContext(), topBarData.getTypeface(), (FrameLayout) layout.findViewById(R.id.game_levelup));
                countdown = new Countdown(game, topBarData.getTypeface(), (TextView) layout.findViewById(R.id.game_countdown));
            }
        });
        pauseButton = (ImageView) layout.findViewById(R.id.game_pausebutton);
        pauseButton.setOnClickListener(this);
        playerProgress = (TextView) layout.findViewById(R.id.game_playerprogress);
        fpsView = (TextView) layout.findViewById(R.id.game_fps);
    }

    public void setScreenSize(Vector screenSize) {
        this.screenSize = screenSize;
    }

    public boolean isScreenSizeSet() {
        return screenSize != null;
    }

    @Override
    public void onClick(View v) {
        if(game.getGameState() == GameState.RUNNING)
        game.pause();
    }

    public void resetToolButtonBar() {
        gameViewHandler.startInUiThread(new Runnable() {
            @Override
            public void run() {
                toolButtonBar.reset();
                toolButtonBar.update(0);
            }
        });
    }

    public void showLevelUp(final int level) {
        gameViewHandler.startInUiThread(new Runnable() {
            @Override
            public void run() {
                levelupMessage.show(level);
            }
        });
    }

    public void showMessage(final boolean wasLastLevel, final MessageType messageType) {
        gameViewHandler.startInUiThread(new Runnable() {
            @Override
            public void run() {
                message.setMessageType(messageType);
                message.fadeIn(wasLastLevel);
            }
        });
    }

    public void showShortMenu() {
        shortMenu.startAnims();
        shortMenu.setVisible(true);
    }

    public void hideShortMenu() {
        shortMenu.setVisible(false);
    }

    public void startCountdown(){
        countdown.start();
    }

    public void stopCountdown(){
        countdown.stop();
    }

    public void update(final boolean gameRunning, final Timer timer, final float playerX) {
        gameViewHandler.startInUiThread(new Runnable() {
            @Override
            public void run() {
                fpsView.setText("Fps: " + timer.getFps());
                if (gameRunning) {
                    playerProgress.setText(((int) (playerX * 20)) / 10f + "m");
                    toolButtonBar.update(timer.getFrameTime());
                }
            }
        });
    }
}
